//
// fp.c
//
// Floating point routines
//
// Copyright (C) 2002 Michael Ringgaard. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
// 1. Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.  
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.  
// 3. Neither the name of the project nor the names of its contributors
//    may be used to endorse or promote products derived from this software
//    without specific prior written permission. 
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
// SUCH DAMAGE.
// 

#define	NDIG	80

#define HI(x)    (*(1 + (int *) &x))
#define LO(x)    (*(int *) &x)
#define HIPTR(x) *(1 + (int *) x)
#define LOPTR(x) (*(int *) x)

static char cvtbuf[NDIG];

#define ANSI_FTOL 1

__declspec(naked) void _ftol() 
{
  __asm    
  {
#if ANSI_FTOL
    fnstcw   word ptr [esp-2]
    mov      ax, word ptr [esp-2]
    or       ax, 0C00h
    mov      word ptr [esp-4], ax
    fldcw    word ptr [esp-4]
    fistp    qword ptr [esp-12]
    fldcw    word ptr [esp-2]
    mov      eax, dword ptr [esp-12]
    mov      edx, dword ptr [esp-8]
#else
    fistp    dword ptr [esp-12]
    mov	     eax, dword ptr [esp-12]
    mov	     ecx, dword ptr [esp-8]
#endif
    ret
  }
}

//
// _ftol2 calls are generated by VC7 compiler to convert doubles to ints
// This routine does not check for special cases
//

__declspec(naked) void _ftol2()
{
  __asm    
  {
    fistp    dword ptr [esp-12]
    mov	     eax, dword ptr [esp-12]
    mov	     ecx, dword ptr [esp-8]
    ret
  }
}

double modf(double x, double *iptr)
{
  int i0, i1, j0;
  unsigned i;

  i0 =  HI(x);
  i1 =  LO(x);
  j0 = ((i0 >> 20) & 0x7ff) - 0x3ff;
  if (j0 < 20) 
  {
    if (j0 < 0) 
    {
      HIPTR(iptr) = i0 & 0x80000000;
      LOPTR(iptr) = 0;
      return x;
    }
    else
    {
      i = (0x000fffff) >> j0;
      if (((i0 & i) | i1) == 0)
      {
	*iptr = x;
	HI(x) &= 0x80000000;
	LO(x)  = 0;
	return x;
      } 
      else
      {
	HIPTR(iptr) = i0 & (~i);
	LOPTR(iptr) = 0;
	return x - *iptr;
      }
    }
  } 
  else if (j0 > 51)
  {
    *iptr = x * 1.0;
    HI(x) &= 0x80000000;
    LO(x) = 0;
    return x;
  }
  else
  {
    i = ((unsigned) (0xffffffff)) >> (j0 - 20);
    if ((i1 & i) == 0) 
    {
      *iptr = x;
      HI(x) &= 0x80000000;
      LO(x) = 0;
      return x;
    }
    else
    {
      HIPTR(iptr) = i0;
      LOPTR(iptr) = i1 & (~i);
      return x - *iptr;
    }
  }
}

static char *cvt(double arg, int ndigits, int *decpt, int *sign, char *buf, int eflag)
{
  int r2;
  double fi, fj;
  char *p, *p1;
  double modf();

  if (ndigits < 0) ndigits = 0;
  if (ndigits >= NDIG - 1) ndigits = NDIG - 2;
  r2 = 0;
  *sign = 0;
  p = &buf[0];
  if (arg < 0)
  {
    *sign = 1;
    arg = -arg;
  }
  arg = modf(arg, &fi);
  p1 = &buf[NDIG];

  if (fi != 0) 
  {
    p1 = &buf[NDIG];
    while (fi != 0) 
    {
      fj = modf(fi / 10, &fi);
      *--p1 = (int)((fj + .03) * 10) + '0';
      r2++;
    }
    while (p1 < &buf[NDIG]) *p++ = *p1++;
  } 
  else if (arg > 0)
  {
    while ((fj = arg * 10) < 1) 
    {
      arg = fj;
      r2--;
    }
  }
  p1 = &buf[ndigits];
  if (eflag == 0) p1 += r2;
  *decpt = r2;
  if (p1 < &buf[0]) 
  {
    buf[0] = '\0';
    return buf;
  }
  while (p <= p1 && p < &buf[NDIG])
  {
    arg *= 10;
    arg = modf(arg, &fj);
    *p++ = (int) fj + '0';
  }
  if (p1 >= &buf[NDIG]) 
  {
    buf[NDIG - 1] = '\0';
    return buf;
  }
  p = p1;
  *p1 += 5;
  while (*p1 > '9') 
  {
    *p1 = '0';
    if (p1 > buf)
      ++*--p1;
    else 
    {
      *p1 = '1';
      (*decpt)++;
      if (eflag == 0) 
      {
	if (p > buf) *p = '0';
	p++;
      }
    }
  }
  *p = '\0';
  return buf;
}

char *ecvt(double arg, int ndigits, int *decpt, int *sign)
{
  return cvt(arg, ndigits, decpt, sign, cvtbuf, 1);
}

char *ecvtbuf(double arg, int ndigits, int *decpt, int *sign, char *buf)
{
  return cvt(arg, ndigits, decpt, sign, buf, 1);
}

char *fcvt(double arg, int ndigits, int *decpt, int *sign)
{
  return cvt(arg, ndigits, decpt, sign, cvtbuf, 0);
}

char *fcvtbuf(double arg, int ndigits, int *decpt, int *sign, char *buf)
{
  return cvt(arg, ndigits, decpt, sign, buf, 0);
}
